package cn.ibizlab.eam.core.ys.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.ys.domain.Student;
import cn.ibizlab.eam.core.ys.filter.StudentSearchContext;
import cn.ibizlab.eam.core.ys.service.IStudentService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.ys.mapper.StudentMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[学生] 服务对象接口实现
 */
@Slf4j
@Service("StudentServiceImpl")
public class StudentServiceImpl extends ServiceImpl<StudentMapper, Student> implements IStudentService {


    private int batchSize = 500;

    @Override
    @Transactional
    public Student get(String key) {
        Student et = getById(key);
        if(et==null){
            et=new Student();
            et.setStudentid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public Student getDraft(Student et) {
        return et;
    }

    @Override
    @Transactional
    public boolean create(Student et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getStudentid()),et);
        return true;
    }

    @Override
    public void createBatch(List<Student> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean save(Student et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(Student et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<Student> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<Student> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Student et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("studentid",et.getStudentid())))
            return false;
        CachedBeanCopier.copy(get(et.getStudentid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<Student> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    public boolean checkKey(Student et) {
        return (!ObjectUtils.isEmpty(et.getStudentid()))&&(!Objects.isNull(this.getById(et.getStudentid())));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<Student> searchDefault(StudentSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<Student> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<Student>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Student> getStudentByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Student> getStudentByEntities(List<Student> entities) {
        List ids =new ArrayList();
        for(Student entity : entities){
            Serializable id=entity.getStudentid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



